/* -*- Mode: C; c-file-style: "bsd"; -*- */

#ifndef __NXP_ELEMENT_H__
#define __NXP_ELEMENT_H__

#include <glib.h>
#include <pthread.h>
#include "nxp_config.h"



typedef struct _NxpElement	NxpElement;
typedef struct _NxpElement	NxpElement_t;
typedef struct _NxpElement	NxpNxpElement;
typedef struct _NxpIdentifier	NxpIdentifier;

/* VisitorFunc is the 'public' callback function, while IteratorFunc is
 * the type implementation (element_type.h) function.  */
typedef void 	    (*NxpElementVisitorFunc)	(NxpElement *elem,
						 gpointer user_data);


#include "eval_context.h"
#include "element_type.h"

/* TODO: rename NxpElementPreferences to NxpPreferences */
typedef struct _NxpElementPreferences NxpElementPreferences;
typedef NxpElementPreferences NxpElementPreferences_t;



struct _NxpElementPreferences 
{
    int number_precision;
};

extern NxpElementPreferences_t nxp_element_preferences;
/* extern gint debug_elements; */
extern gint nxp_element_trace_references;
//extern volatile gint nxp_element_global_count;
//extern volatile gint nxp_element_references_count;


#ifdef NXP_ELEMENT_DEBUG

NxpElement *nxp__element_typecast_and_sanity_check(gpointer p);
# define NXP_ELEMENT(x) nxp__element_typecast_and_sanity_check(x)

#else

# define NXP_ELEMENT(x) ((NxpElement *)(x))

#endif

#define NXP_ELEMENT_GET_TYPE_INFO(x) (NXP_ELEMENT(x)->type_info)
#define NXP_ELEMENT_GET_TYPE(x) (GPOINTER_TO_INT(NXP_ELEMENT(x)->type_info->type))

#define NXP_ELEMENT_GET_TYPE_NAME(el) ((el)? (NXP_ELEMENT_GET_TYPE_INFO(el)->name) :\
                                       "(null element)")

NxpElement* nxp__element_check_type(NxpElement *el, NxpElementType type);

#ifdef NXP_ELEMENT_DEBUG

# define NXP_ELEMENT_TYPECAST(el, type, c_type) ((c_type *)nxp__element_check_type(NXP_ELEMENT(el), type))

#else

# define NXP_ELEMENT_TYPECAST(el, type, c_type) ((c_type *)(el))

#endif

//#define NXP_ELEMENT_UTREE(x)	(NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_UTREE,	NxpElementUTree	))


#ifdef NXP_ELEMENT_DEBUG

void nxp__element_ref	(NxpElement *el, const char *file, const char *function, int line);
void nxp__element_unref (NxpElement *el, const char *file, const char *function, int line);


typedef struct {
    gboolean ref:1;
    const char *function;
    const char *file;
    gint line;
} NxpElementRefTraceItem;


/* Reference element; returns the element; if element is NULL, print a
   warning; if el is NULL and debug_elements is true, also issue a
   breakpoint */
#define NXP_ELEMENT_REF(el)	(nxp__element_ref(NXP_ELEMENT(el), __FILE__,\
						  __PRETTY_FUNCTION__, __LINE__), (el))

/* Unreference element; returns NULL; if element is NULL, print a
   warning; if el is NULL and debug_elements is true, also issue a
   breakpoint */
#define NXP_ELEMENT_UNREF(el) (nxp__element_unref(NXP_ELEMENT(el), __FILE__,\
						  __PRETTY_FUNCTION__, __LINE__), NULL)


#else 

/* Reference element; returns the element */
#define NXP_ELEMENT_REF(el)	({ g_atomic_int_inc(&NXP_ELEMENT(el)->ref_count); (el); })
/* Unreference element; returns NULL */
#define NXP_ELEMENT_UNREF(el)	({ if (g_atomic_int_dec_and_test(&NXP_ELEMENT(el)->ref_count))  \
                                       nxp__element_destroy((NxpElement*)(el));                 \
                                       NULL; })

#endif

#define NXP_ELEMENT_XREF(el) if (el) NXP_ELEMENT_REF(el)
#define NXP_ELEMENT_XUNREF(el) if (el) NXP_ELEMENT_UNREF(el)



/* ----- NxpElement ------ */

struct _NxpElement
{
    NxpElementTypeInfo *type_info;
    volatile gint ref_count;
#ifdef NXP_ELEMENT_DEBUG
    guint16 magic;
    GSList *ref_trace;
#endif
};



gboolean    nxp_element_check_type		      (NxpElement *el,
						       int type);
void        nxp_elements_init                         (void);
void        nxp_elements_destroy                      (void);
NxpElement* nxp_element_alloc                         (NxpElementTypeInfo      *type_info,
						       size_t                   size);
void        nxp__element_destroy                      (NxpElement              *el);
void        nxp_element_foreach                       (NxpElement              *elem,
                                                       NxpElementVisitorFunc    func,
                                                       gpointer                 user_data,
                                                       NxpEvalContext          *context);
NxpElement* nxp_element_derive                        (NxpElement              *el,
						       NxpIdentifier           *var,
						       NxpEvalContext          *context,
						       GError                 **error);
NxpElement* nxp_element_derive_simple                 (NxpElement              *el,
						       const char              *var,
						       NxpEvalContext          *context);
void        nxp_element_abort_thread                  (pthread_t                tid);
gboolean    nxp_element_check_thread_abortion_request (void);
NxpElement* nxp_element_eval                          (NxpElement              *el,
						       NxpEvalContext          *context,
						       GError                 **error);
gboolean    nxp_element_depends_on		      (NxpElement	       *self,
						       NxpElement	       *variable,
						       NxpEvalContext	       *context,
						       GError		     **error);
NxpElement* nxp_element_simplify                      (NxpElement              *el,
						       NxpEvalContext          *context,
						       GError                 **error);
NxpElement* nxp_element_substitute		      (NxpElement	       *elem,
						       NxpElement	       *var,
						       NxpElement	       *el,
						       NxpEvalContext	       *context,
						       GError		      **error);
gboolean    nxp_element_is_true                       (NxpElement              *element,
						       NxpEvalContext          *context,
						       GError                 **error);
NxpElement* nxp_element_ref                           (NxpElement              *elem);
NxpElement* nxp_element_unref                         (NxpElement              *elem);

void nxp_element_data_unref (gpointer data);



/*  gboolean nxp_element_is_scalar		       (NxpElement	       *el); */
/*  NxpElement* nxp_element_substitute_variable		  (NxpElement		  *el, */
/*						NxpIdentifier	       *var_name, */
/*						NxpElement	       *sub_tree); */
/*  gboolean nxp_element_is_const		       (NxpElement	       *el); */



#endif /* #ifndef __NXP_ELEMENT_H__ */





