/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#ifndef __ELEMENT_TYPE_H__
#define __ELEMENT_TYPE_H__

#define NXP_ELEMENT_TYPE(x) x->type_info->type

/* forward declarations */
typedef struct _NxpElementTypeInfo	NxpElementTypeInfo;
typedef        int                      NxpElementType;

#define NXP_ELEMENT_TYPE_ANY  ((guint16)0xffff)
#define NXP_ELEMENT_TYPE_NONE ((guint16)0)

/* -------------------- */
#include "identifier.h"
#include "element.h"
/* -------------------- */

typedef void        (*NxpElementDestroyFunc)	(NxpElement *elem);
typedef GString*    (*NxpElementReprFunc)	(NxpElement *elem, int *precedence);
typedef GString*    (*NxpElementStrFunc)	(NxpElement *elem, int *precedence);
typedef NxpElement* (*NxpElementEvalFunc)	(NxpElement *elem, 
						 NxpEvalContext *context,
						 GError **error);
typedef NxpElement* (*NxpElementSimplifyFunc)	(NxpElement *elem, 
						 NxpEvalContext *context,
						 GError **error);
typedef NxpElement* (*NxpElementDeriveFunc)	(NxpElement *elem,
						 NxpIdentifier *var,
						 NxpEvalContext *context,
						 GError **error);
typedef gboolean    (*NxpElementTruthFunc)	(NxpElement *elem, 
						 NxpEvalContext *context, 
						 GError **error);
typedef gboolean    (*NxpElementDependsonFunc)	(NxpElement *elem,
						 NxpElement *var,
						 NxpEvalContext *context, 
						 GError **error);
typedef NxpElement* (*NxpElementSubstituteFunc)	(NxpElement *elem,
						 NxpElement *var,
						 NxpElement *el,
						 NxpEvalContext *context,
						 GError **error);

/* This function should recursively visit all elements contained in
 * elem. */
typedef void 	    (*NxpElementIteratorFunc)	(NxpElement *elem,
                                                 NxpElementVisitorFunc func,
						 gpointer user_data,
                                                 NxpEvalContext *context);

struct _NxpElementTypeInfo {
    NxpElementTypeInfo	       *parent;
    NxpElementType		type;
    GHashTable                 *user_data;
    /*< public >*/
    char		       *name;
    NxpElementDestroyFunc	destroy_func;
    NxpElementEvalFunc		eval_func;
    NxpElementSimplifyFunc	simplify_func;
    NxpElementDeriveFunc	derive_func;
    NxpElementIteratorFunc	iterator_func;
    NxpElementReprFunc		repr_func;
    NxpElementStrFunc		str_func;
    NxpElementTruthFunc		truth_func;
    NxpElementDependsonFunc	dependson_func;
    NxpElementSubstituteFunc	substitute_func;
};


extern int          nxp_element_last_type;

void                nxp_element_type_init	   (void);
void                nxp_element_type_fini	   (void);
NxpElementTypeInfo* nxp_element_type_register_info (NxpElementType type,
						    guint	   size,
						    NxpElementType parent,
						    guint	   parent_size);
NxpElementTypeInfo* nxp_element_type_get_info	   (int type);

int                 nxp_element_next_type	   (void);

gpointer            nxp_element_type_get_data	(NxpElementTypeInfo  *type_info,
						 const char* property_name);

void                nxp_element_type_set_data (NxpElementTypeInfo  *type_info,
					       const char* property_name,
					       gpointer data);

GSList*		    nxp_element_type_get_children_types(NxpElementType type);

#endif /* __ELEMENT_TYPE_H__ */
