#ifndef __NXPINTPOLY_H__
#define __NXPINTPOLY_H__

#include <gmp.h>
#include <glib.h>
#include "element.h"
#include "types/int.h"

#define		NXP_INT_POLY(obj)   ((NxpIntPoly)obj)

typedef struct _NxpIntPoly  NxpIntPoly;

struct _NxpIntPoly {
    mpz_t *coeff;
    int deg;
    gint lmax;
};

void nxp_int_poly_init_set_mpoly		    (NxpIntPoly *r,
						     NxpElement *p,
						     GError **error);
void	    nxp_int_poly_init			    (NxpIntPoly *r, gint size);
void	    nxp_int_poly_set			    (NxpIntPoly *res,
						     NxpIntPoly *a);
void	    nxp_int_poly_view			    (NxpIntPoly *p);
void	    nxp_int_poly_init_set		    (NxpIntPoly *res,
						     NxpIntPoly *a);

NxpElement* nxp_element_mpoly_new_from_int_poly	    (NxpIntPoly *a,
						     int	size,
						     NxpElement *vars);
void	    nxp_int_poly_clear			    (NxpIntPoly *p);
void	    nxp_int_poly_derive			    (NxpIntPoly *r,
						     NxpIntPoly *a);
GSList *    nxp_int_poly_sff			    (NxpIntPoly *p);
void	    nxp_int_poly_normalize		    (NxpIntPoly *r,
						     NxpIntPoly *a, mpz_t n);
void	    nxp_int_poly_normalize_ui		    (NxpIntPoly *r,
						     NxpIntPoly *a, guint n);
void	    nxp_int_poly_mul			    (NxpIntPoly *r,
						     NxpIntPoly *p1,
						     NxpIntPoly *p2);
void	    nxp_int_poly_add			    (NxpIntPoly *r,
						     NxpIntPoly *p1,
						     NxpIntPoly *p2);
void	    nxp_int_poly_sub			    (NxpIntPoly *r,
						     NxpIntPoly *p1,
						     NxpIntPoly *p2);
void	    nxp_int_poly_power			    (NxpIntPoly *res,
						     NxpIntPoly *a,
						     int p);
void	    nxp_int_poly_mignotte		    (mpz_t bound,
						     NxpIntPoly *a,
						     int n);
void	    nxp_int_poly_gcd			    (NxpIntPoly *r,
						     NxpIntPoly *a,
						     NxpIntPoly *b);
void	    nxp_int_poly_ezgcd			    (NxpIntPoly *gcd,
						     NxpIntPoly *f,
						     NxpIntPoly *g);
void	    nxp_int_poly_realloc		    (NxpIntPoly *a,
						     int newsize);
void	    nxp_int_poly_set			    (NxpIntPoly *res,
						     NxpIntPoly *a);
void	    nxp_int_poly_divexact_mpz		    (NxpIntPoly *res,
						     NxpIntPoly *a,
						     mpz_t b);
void	    nxp_int_poly_mul_mpz		    (NxpIntPoly *res,
						     NxpIntPoly *a,
						     mpz_t b);
void	    nxp_int_poly_mul_si			    (NxpIntPoly *res,
						     NxpIntPoly *a,
						     nxp_int_t b);
void	    nxp_int_poly_sub_si			    (NxpIntPoly *res,
						     NxpIntPoly *a,
						     nxp_int_t b);
void	    nxp_int_poly_cont			    (mpz_t c,
						     NxpIntPoly *a);
gboolean    nxp_int_poly_divexact		    (NxpIntPoly *quo,
						     NxpIntPoly *a,
						     NxpIntPoly *b);
GSList *    nxp_int_poly_build_factors		    (NxpIntPoly *pol,
						     GArray *lift, mpz_t p);
#endif /* __NXPINTPOLY_H__ */
