/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#ifndef __NXP_PROTOTYPES_H__
#define __NXP_PROTOTYPES_H__

#include <glib.h>
#include "element.h"
#include "identifier.h"

typedef void        (*NxpFreeData)              (gpointer user_data);

typedef NxpElement* (*NxpEvalMarshal) 		(gpointer function,
						 GSList *args,
						 GHashTable *named_args,
						 gpointer user_data,
				       		 NxpEvalContext *context,
						 GError **error);

typedef NxpElement* (*NxpSimplifyMarshal) 	(gpointer function,
						 GSList *args,
						 GHashTable *named_args,
						 gpointer user_data,
					   	 NxpEvalContext *context,
						 GError **error);

typedef NxpElement* (*NxpDeriveMarshal) 	(gpointer function,
						 NxpIdentifier *var,
						 GSList *args,
						 GHashTable *named_args,
						 gpointer user_data,
						 NxpEvalContext *context,
						 GError **error);

typedef struct _NxpProtoClass NxpProtoClass;
typedef struct _NxpProtoInstance NxpProtoInstance;

typedef enum {
    NXP_PROTO_INSTANCE_DETAILED = (1<<0),
    NXP_PROTO_INSTANCE_VOLATILE = (1<<1),
} NxpProtoInstanceFlags;



struct _NxpProtoClass
{
    int                 arity;
    NxpElementType     *types;
    NxpEvalMarshal      eval_marshal;
    NxpSimplifyMarshal  simplify_marshal;
    NxpDeriveMarshal    derive_marshal;
};


struct _NxpProtoInstance
{
    const NxpProtoClass   *protoclass;
    NxpProtoInstanceFlags  flags;
    gpointer               user_data;
    NxpFreeData            free_user_data;
    gpointer               eval_func;
    gpointer               derive_func;
    gpointer               simplify_func;
};


NxpProtoInstance* nxp_proto_instance_new     (void);
void              nxp_proto_instance_free    (NxpProtoInstance       *proto);
void              nxp_bifunc_define          (NxpIdentifier          *ident,
					      const NxpProtoClass    *prototype,
					      gpointer                eval_func,
					      gpointer                derive_func,
					      gpointer                simplify_func,
					      NxpEvalContext         *context,
					      GError                **error);
void              nxp_bifunc_define_full     (NxpIdentifier          *ident,
					      const NxpProtoClass    *prototype,
					      gpointer                eval_func,
					      gpointer                derive_func,
					      gpointer                simplify_func,
					      gpointer                user_data,
					      NxpFreeData             free, 
					      NxpEvalContext         *context,
					      GError                **error);
NxpDefinition*    nxp_bifunc_define_detailed(NxpIdentifier          *aident,
					      const NxpProtoClass    *prototype,
					      gpointer                eval_func,
					      gpointer                derive_func,
					      gpointer                simplify_func,
					      gpointer                user_data,
					      NxpFreeData             free_data,
					      NxpEvalContext         *context,
					      NxpProtoInstanceFlags   flags,
					      GError                **error);
void              nxp_bifunc_define_global   (const char             *name,
					      const NxpProtoClass    *prototype,
					      gpointer                eval_func,
					      gpointer                derive_func,
					      gpointer                simplify_func,
					      NxpEvalContext         *context,
					      GError                **error);
void            nxp_bifunc_define_global_full(const char             *name,
					      const NxpProtoClass    *prototype,
					      gpointer                eval_func,
					      gpointer                derive_func,
					      gpointer                simplify_func,
					      gpointer                user_data,
					      NxpFreeData             free_data,
					      NxpEvalContext         *context,
					      GError                **error);
NxpProtoInstance* nxp_find_eval_prototype(NxpElement *el_args,
					  NxpElement **new_args,
					  GSList *proto_list,
					  NxpEvalContext *context, GError **error);
NxpProtoInstance* nxp_find_simp_prototype(NxpElement *el_args,
					  NxpElement **new_args,
					  GSList *proto_list,
					  NxpEvalContext *context, GError **error);


void nxp_bifunc_define_global_volatile(const char *name, const NxpProtoClass *prototype,
				       gpointer eval_func, gpointer derive_func,
				       gpointer simplify_func, NxpEvalContext *context, 
				       GError **error);

void nxp_bifunc_define_global_full_volatile(const char *name, const NxpProtoClass *prototype,
					    gpointer eval_func, gpointer derive_func,
					    gpointer simplify_func, gpointer user_data,
					    NxpFreeData free_data,
					    NxpEvalContext *context, GError **error);



/* =========================================================================== */
/*                              STANDARD PROTOTYPES                            */
/* =========================================================================== */

#include "prototypes.generated.h"


#endif /* __NXP_PROTOTYPES_H__ */


