#ifndef __MATRIX_H__
#define __MATRIX_H__


#include "element.h"
#include "vector.h"

typedef struct _NxpElementMatrix	NxpElementMatrix;

#define NXP_ELEMENT_TYPE_MATRIX	(nxp_element_matrix_get_type())
#define NXP_ELEMENT_MATRIX(x)	(NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_MATRIX,	NxpElementMatrix	))
#define NXP_ELEMENT_IS_MATRIX(x)	        nxp_element_check_type((x), NXP_ELEMENT_TYPE_MATRIX)

struct _NxpElementMatrix {
    NxpElement element;
    guint width, height;
    NxpElement **data;
};

#define NXP_ELEMENT_MATRIX_WIDTH(a)  a->width
#define NXP_ELEMENT_MATRIX_HEIGHT(a) a->height

#define NXP_ELEMENT_MATRIX_IS_SQUARE(a) ({ NxpElementMatrix *m = NXP_ELEMENT_MATRIX(a); \
                                       m->width == m->height; })

#define NXP_ELEMENT_MATRIX_ELEM_YX(x, row, col)   (x->data[(row) * (x)->width + (col)])
#define NXP_ELEMENT_MATRIX_ELEM_XY(x, col, row)   (x->data[(row) * (x)->width + (col)])
/* returns an lvalue of an element of the matrix -- should not be used
   directly (usually); element should be an NxpElementMatrix*  */
#define _element_matrix_index(elem, row, col) elem->data[(row) * (elem)->width + (col)]


int	    nxp_element_matrix_get_type	   (void);
void        nxp_element_matrix_type_init   (void);
NxpElement* nxp_element_matrix_new         (guint              width,
					    guint              height,
					    NxpElement       **data);
void        _element_matrix_set            (NxpElementMatrix  *matrix,
					    guint              row,
					    guint              col,
					    NxpElement        *element);
void        _element_matrix_set_row        (NxpElementMatrix  *matrix,
					    guint              row,
					    NxpElement        *element);
void        _element_matrix_set_col        (NxpElementMatrix  *matrix,
					    guint              col,
					    NxpElement        *element);
NxpElement* _element_matrix_get            (NxpElementMatrix  *matrix,
					    guint              row,
					    guint              col);
NxpElement* nxp_element_matrix_real_put    (NxpElementMatrix  *matrix,
					    NxpElement        *element,
					    guint              row,
					    guint              col);
NxpElement* nxp_element_matrix_determinant (NxpElementMatrix  *mat,
					    NxpEvalContext    *context,
					    GError	      **error);
NxpElement* matrix_times_scalar            (NxpElementMatrix  *mat,
					    NxpElement        *scalar,
					    NxpEvalContext    *context,
					    GError           **error);
NxpElement *vector_times_matrix(NxpElementVector *v, NxpElementMatrix *m,
				NxpEvalContext *context, GError **error);
NxpElement *matrix_times_vector(NxpElementMatrix *m, NxpElementVector *v, 
				NxpEvalContext *context, GError **error);
/*  NxpElement* matrix_times_matrix            (NxpElementMatrix  *mat1, */
/*  					    NxpElementMatrix  *mat2); */




















#if 0

/* Construct matrix */
void       matrix_construct (Matrix    *matrix,
			     int        i,
			     int        j);



//void matrix_destroy(Matrix* vector);
/* Clean de matrix. If recursive, destroy it's child elements */
void       matrix_clean     (Matrix    *matrix,
			     gboolean   recursive);
void       matrix_put       (Matrix    *vector,
			     NxpElement   *element,
			     int        i,
			     int        j);
NxpElement*  matrix_get        (const Matrix    *vector,
			     int        i,
			     int        j);



//Matrix* matrix_copy(Matrix* matrix);
/* Copy matrix 'src' into 'matrix 'dest'. 'dest' is assumed to be
   clean -- it will be overwritten. */
void       matrix_copy      (Matrix    *dest,
			     const Matrix    *src);
void       matrix_det_cb    (void);
void       matrix_det_cb_ok (GtkWidget *vbox);
NxpElement*   MatrixDet        (Matrix    *mat);
GtkWidget* MatrixShow       (Matrix    *mat);
Matrix*    matrix_expand    (Matrix* m,int i, int j);
void       matrix_fill      (Matrix* z);
Matrix*    matrix_real_put  (Matrix* matrix, NxpElement* element, int i, int j) ;

#endif


#endif

