#ifndef __MPOLY_H__
#define __MPOLY_H__

#include "int.h"
#include "vector.h"
#include "list.h"
#include "element.h"

typedef struct _NxpElementMpoly		NxpElementMpoly;
typedef struct _NxpElementMpolyTypeInfo		NxpElementMpolyTypeInfo;

/* The monomial 5*x^2*y^3 is represented by the vector {5, {2, 3}} */

#define NXP_MONOMIAL_VECTOR(v)	_element_vector_index(NXP_ELEMENT_VECTOR(v), 1)
#define NXP_MONOMIAL_COEFF(v)	_element_vector_index(NXP_ELEMENT_VECTOR(v), 0)

#define NXP_ELEMENT_TYPE_MPOLY (nxp_element_mpoly_get_type())
#define NXP_ELEMENT_MPOLY(x)	(NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_MPOLY,	NxpElementMpoly))
#define NXP_ELEMENT_IS_MPOLY(x)	        nxp_element_check_type((x), NXP_ELEMENT_TYPE_MPOLY)

struct _NxpElementMpoly {
    NxpElement		parent;
    NxpElementList	*mon; /* Monomials sorted from the smallest degree to
				 the highest one. */
    NxpElementList	*args; /* Different variables used in the polynomial */
};

struct _NxpElementMpolyTypeInfo {
    NxpElementTypeInfo	parent;
};

/************************* Functions declarations ****************************/
NxpElement *nxp_monomial_new		    (NxpElement *coeff, NxpElement *v);

NxpElement *nxp_element_mpoly_insert	    (NxpElement *p, NxpElement *el,
					     int index);
NxpElement *nxp_element_mpoly_opposed	    (NxpElement *p);
NxpElement *nxp_element_mpoly_euclidean	    (NxpElement *p1, NxpElement *p2,
					     NxpEvalContext *context,
					     GError **error);

int	    nxp_element_mpoly_get_type	    (void);
void	    nxp_element_mpoly_type_init	    ();
void	    nxp_element_mpoly_number_init   (GError		 **error);
void	    nxp_element_mpoly_variable_init (GError		 **error);

NxpElement *nxp_element_mpoly_new(NxpElement *mon, NxpElement *args);
NxpElement *nxp_element_mpoly_new_simple(NxpElement *vector, NxpElement *args);
NxpElement *nxp_element_mpoly_add_monomial(NxpElement *p, NxpElement *m, NxpEvalContext *context, GError **error);
void nxp_element_mpoly_function_init		(NxpEvalContext *context,
						 GError **error);
#endif
