/* -*- Mode: C; c-file-style: "bsd"; -*- */
#ifndef __TREE_H__
#define __TREE_H__

typedef struct _NxpElementTree 	NxpElementTree;

#include "element.h"
#include "op_bindings.h"

#define NXP_ELEMENT_TYPE_TREE	(nxp_element_tree_get_type())
#define NXP_ELEMENT_TREE(x)		(NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_TREE,	NxpElementTree	))
#define NXP_ELEMENT_IS_TREE(x)	        nxp_element_check_type((x), NXP_ELEMENT_TYPE_TREE)




struct _NxpElementTree {
    NxpElement element;
    NxpOp op;			/* operator (unary or binary) */
    NxpElement *left;		/* should be null for unary left-side operators */
    NxpElement *right;		/* should be null for unary right-side operators */
};

int	    nxp_element_tree_get_type	(void);
void        nxp_element_tree_type_init	(void);
NxpElement* nxp_element_tree_new	(NxpOp        op,
					 NxpElement  *left,
					 NxpElement  *right);
NxpElement* nxp_element_tree_simplify_op(NxpOp        op,
					 NxpElement  *left,
					 NxpElement  *right,
					 NxpEvalContext *context, 
					 GError     **error);
NxpElement* nxp_element_tree_eval_op	(NxpOp        op,
					 NxpElement  *left,
					 NxpElement  *right,
					 NxpEvalContext *context, 
					 GError     **error);
gboolean nxp_element_eq			(NxpElement * a,
					 NxpElement * b,
					 NxpEvalContext * context,
					 GError ** error);

/* this macro unfortunately requires gcc; TODO: figure out an
   alternative for other compilers. It would have to be a C
   function... */
#define elem_BINARY(tree_type, a, b)				\
({								\
    NxpElement *el1 = NXP_ELEMENT(a), *el2 = NXP_ELEMENT(b), *result;	\
    result = nxp_element_tree_new(tree_type, el1, el2);		\
    NXP_ELEMENT_UNREF(el1);						\
    NXP_ELEMENT_UNREF(el2);						\
    result;							\
})

#define elem_UNARY(tree_type, a, b)				\
({								\
    NxpElement *result, *el1 = NULL, *el2 = NULL;               \
    if (a) el1 = NXP_ELEMENT(a);                                \
    if (b) el2 = NXP_ELEMENT(b);                                \
    result = nxp_element_tree_new(tree_type, el1, el2);		\
    if (el1) NXP_ELEMENT_UNREF(el1);				\
    if (el2) NXP_ELEMENT_UNREF(el2);				\
    result;							\
})



NxpElement* elem_UMINUS(NxpElement* el);

#define elem_NOT(a)		elem_UNARY(NXP_OP_TILDE_LEFT, NULL, a)
#define elem_FACTORIAL(a)	elem_UNARY(NXP_OP_EXCL_RIGHT, a, NULL)

#define elem_PLUS(a, b)		elem_BINARY(NXP_OP_PLUS, a, b)
#define elem_MINUS(a, b)	elem_BINARY(NXP_OP_MINUS, a, b)
#define elem_TIMES(a, b)	elem_BINARY(NXP_OP_STAR, a, b)
#define elem_OVER(a, b)		elem_BINARY(NXP_OP_SLASH, a, b)
#define elem_PERCENT(a, b)      elem_BINARY(NXP_OP_PERCENT, a, b)
#define elem_CIRC(a, b)		elem_BINARY(NXP_OP_HAT, a, b)
#define elem_LT(a, b)		elem_BINARY(NXP_OP_LT, a, b)
#define elem_LE(a, b)		elem_BINARY(NXP_OP_LT_EQ, a, b)
#define elem_GT(a, b)		elem_BINARY(NXP_OP_GT, a, b)
#define elem_GE(a, b)		elem_BINARY(NXP_OP_GT_EQ, a, b)
#define elem_NE(a, b)		elem_BINARY(NXP_OP_EXCL_EQ, a, b)
#define elem_EQ(a, b)		elem_BINARY(NXP_OP_EQ2, a, b)
#define elem_AND(a, b)		elem_BINARY(NXP_OP_SLASH_BSLASH, a, b)
#define elem_OR(a, b)		elem_BINARY(NXP_OP_BSLASH_SLASH, a, b)
#define elem_DOT3(a, b)		elem_BINARY(NXP_OP_DOT3, a, b)


#define _eval_plus(a, b) 	nxp_element_tree_eval_op(NXP_OP_PLUS, a, b, context, NULL)
#define _eval_minus(a, b)	nxp_element_tree_eval_op(NXP_OP_MINUS, a, b, context, NULL)
//#define _eval_uminus(a, b)	nxp_element_tree_eval_op(NXP_OP_MINUS_LEFT, NULL, a, context, NULL)
#define _eval_over(a, b)	nxp_element_tree_eval_op(NXP_OP_SLASH, a, b, context, NULL)
#define _eval_times(a, b)	nxp_element_tree_eval_op(NXP_OP_STAR, a, b, context, NULL)
#define _eval_circ(a, b)	nxp_element_tree_eval_op(NXP_OP_HAT, a, b, context, NULL)
#define _eval_atrib(a, b)	nxp_element_tree_eval_op(NXP_OP_EQ, a, b, context, NULL)
#define _eval_store(a, b)	nxp_element_tree_eval_op(NXP_OP_COLON_EQ, a, b, context, NULL)
#define _eval_eq(a, b)		nxp_element_tree_eval_op(NXP_OP_EQ2, a, b, context, NULL)
#define _eval_neq(a, b)		nxp_element_tree_eval_op(NXP_OP_EXCL_EQ, a, b, context, NULL)
#define _eval_lt(a, b)		nxp_element_tree_eval_op(NXP_OP_LT, a, b, context, NULL)
#define _eval_le(a, b)		nxp_element_tree_eval_op(NXP_OP_LT_EQ, a, b, context, NULL)
#define _eval_gt(a, b)		nxp_element_tree_eval_op(NXP_OP_GT, a, b, context, NULL)
#define _eval_ge(a, b)		nxp_element_tree_eval_op(NXP_OP_GT_EQ, a, b, context, NULL)
#define _eval_not(a)		nxp_element_tree_eval_op(NXP_OP_TILDE_LEFT, NULL,  a, context, NULL)
#define _eval_and(a, b)		nxp_element_tree_eval_op(NXP_OP_SLASH_BSLASH, a, b, context, NULL)
#define _eval_or(a, b)		nxp_element_tree_eval_op(NXP_OP_BSLASH_SLASH, a, b, context, NULL)

#endif /* __TREE_H__ */






