#ifndef __VECTOR_H__
#define __VECTOR_H__

#include "int.h"
#include "element.h"

typedef struct _NxpElementVector	NxpElementVector;

#define NXP_ELEMENT_TYPE_VECTOR	(nxp_element_vector_get_type())
#define NXP_ELEMENT_VECTOR(x)	(NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_VECTOR,	NxpElementVector	))
#define NXP_ELEMENT_IS_VECTOR(x)	        nxp_element_check_type((x), NXP_ELEMENT_TYPE_VECTOR)

struct _NxpElementVector {
    NxpElement   element;
    guint        size;
    NxpElement **data;
};

#define NXP_ELEMENT_VECTOR_SIZE(x)    x->size
#define NXP_ELEMENT_VECTOR_ELEM(v, pos)  ((NxpElement **)v->data)[pos]

/* returns an lvalue of an element of the vector -- should not be used
   directly (usually); element should be an NxpElementVector*  */
#define _element_vector_index(elem, i) elem->data[i]

int	    nxp_element_vector_get_type	(void);
void        nxp_element_vector_type_init();
NxpElement* nxp_element_vector_new      (guint              size,
					 NxpElement       **data);
NxpElement* nxp_element_vector_new_from_range(NxpElementInt* low, NxpElementInt* high, GError **error);
NxpElement* nxp_element_vector_extract_range(NxpElement *v, int from, int to);
void        _element_vector_set         (NxpElementVector  *vec,
					 guint              index,
					 NxpElement        *element);
NxpElement* nxp_element_vector_insert	(NxpElement	   *vector,
					 guint		    index,
					 NxpElement	   *element);
NxpElement* _element_vector_get         (NxpElementVector  *vec,
					 guint              index);
NxpElement* nxp_element_vector_real_put (NxpElementVector  *vector,
					 NxpElement        *element,
					 guint              index);
NxpElement* vector_times_vector         (NxpElementVector  *vec1,
					 NxpElementVector  *vec2,
					 NxpEvalContext    *context);
NxpElement* _element_vector_mul_shl	(NxpElement *v,
					 NxpElement *n,
					 int sh,
					 NxpEvalContext *context,
					 GError **error);
NxpElement* nxp_element_vector_set_size	(NxpElement *v, guint size);

gboolean nxp_element_vector_made_of(NxpElement *el, NxpElementType type);
gint nxp_element_vector_int_compare(NxpElement *a, NxpElement *b);
gint nxp_element_vector_compare(NxpElement *a, NxpElement *b);

#endif
